package data_access;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.BiFunction;
import java.util.function.Function;

class DataRepositoryImpl<T,ID> implements DataRepository<T,ID> {
    private final List<T> list;
    private final Function<T, ID> getId; // T getId( ID id );
    private final BiFunction<ID, ID, Boolean> compareId; // Boolean compareId( ID id, ID id2 );

    /**
     * Constructor.
     *
     * @param getId     functional interface to obtain id from entity
     * @param compareId functional interface to compare two entity id's
     */
    DataRepositoryImpl(
            Function<T, ID> getId,
            BiFunction<ID, ID, Boolean> compareId
    ) {
        this.list = new ArrayList<T>();
        this.getId = getId;
        this.compareId = compareId;
    }


    // Add entity to list, only if entity with same ID is not yet present in
    // list. If ID is present, replace entity in list. Return saved entity.
    @Override
    public <S extends T> S save(S entity) {
        if (entity != null) {
            for (int i = 0; i < list.size(); i++) {
                T entity1 = list.get(i);
                ID id1 = getId.apply(entity1);
                ID entitiyPara = getId.apply(entity);

                // Ersetzen:
                if (compareId.apply(entitiyPara, id1)) {
                    list.add(i, entity);
                    return entity;
                }
            }
            // wenn noch nicht vorhanden:
            list.add(entity);
        }
        return entity;
    }


    @Override
    public <S extends T> Iterable<S> saveAll(Iterable<S> entities) {
        if(entities != null){
            for(S current : entities) {
                this.save(current);
            }
        }
        return entities;
    }

    @Override
    public Optional<T> findById(ID id) {
        if(id != null) {
            for (int i = 0; i < this.list.size(); i++) {
                T entity = list.get(i);
                ID idFind = getId.apply(entity);
                if(id.equals(idFind)){
                    Optional<T> result = Optional.ofNullable(entity);
                    return result;
                }
            }
        }
        return Optional.empty();
    }

    @Override
    public boolean existsById(ID id) {
        if(id != null){
            for(int i = 0; i < this.list.size();i++ ){
                T entity = list.get(i);
                ID idFind = getId.apply(entity);
                if(compareId.apply(id, idFind)){
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public Iterable<T> findAll(){
        return this.list;
    }

    @Override
    public Iterable<T> findAllById(Iterable<ID> ids) {
        ArrayList<T> result = new ArrayList<>();
        if(ids != null){
            for(ID current : ids){
                int index = this._find(current);
                if (index != -1) {
                    result.add(this.list.get(index));
                }
            }
        }
        return result;
    }

    @Override
    public long count() {
        return list.size();
    }

    @Override
    public void deleteById(ID id) {
        if(id != null){
            for(int i = 0; i < this.list.size(); i++){
                T entity = list.get(i);
                ID id2 = getId.apply(entity);

                if (compareId.apply(id, id2)) {
                    this.delete(entity);
                }
            }
        }
    }

    @Override
    public void delete(T entity) {
        if (entity != null) {
            for (int i = 0; i < list.size(); i++) {
                T entity1 = list.get(i);
                if(entity.equals(entity1)){
                    list.remove(entity);
                }
            }
        }
    }

    @Override
    public void deleteAll(Iterable<? extends T> entities) {
        if(entities != null){
            for(T current : entities){
                if(this.list.indexOf(current) != -1){
                    delete((T) entities);
                }
            }
        }

    }

    @Override
    public void deleteAll() {
        list.clear();
    }

    // Returns object index in list, if object with
    private int _find(ID id) {
        if (id != null) {
            int len = list.size();
            for (int i = 0; i < len; i++) {
                T entity = list.get(i);
                ID id2 = getId.apply(entity);

                if (compareId.apply(id, id2)) {
                    return i;

                }
            }
        }
        return -1;
    }

    @Override
    public void startup() {

    }

    @Override
    public void shutdown() {

    }
}