package system.builder;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;

import data_access.DataAccess;
import model.Article;
import model.Customer;
import model.Order;
import model.OrderItem;


/**
 * Local singleton instance that implements the DataAccess.MockDataBuilder
 * interface and that fills repositories with some mock data.
 * 
 * @author sgra64
 *
 */
class MockDataBuilderImpl implements MockDataBuilder {

	private final DataAccess dataAccess;

	private Customer cEric, cAnne, cNadine, cTimo, cSandra;

	private Article a1, a2, a3, a4, a5, a6;

	private static final Object[][] priceTable_Normal = new Object[][] {
		{ "SKU-458362",  299 },	// Tasse
		{ "SKU-693856",  149 },	// Becher
		{ "SKU-518957", 2000 },	// Kanne
		{ "SKU-638035",  649 },	// Teller
		{ "SKU-386035", 2999 },	// Kaffeemaschine
		{ "SKU-443803", 1999 },	// Teekocher
	};

	@SuppressWarnings("unused")
	private static final Object[][] priceTable_Discount = new Object[][] {
		{ "SKU-458362",  249 },	// Tasse
		{ "SKU-693856",   99 },	// Becher
		{ "SKU-518957", 1499 },	// Kanne
		{ "SKU-638035",  499 },	// Teller
		{ "SKU-386035", 2499 },	// Kaffeemaschine
		{ "SKU-443803", 1499 },	// Teekocher
	};

	private Object[][] priceTable = priceTable_Normal;



	/*
	 * Local constructor.
	 */
	public MockDataBuilderImpl( DataAccess dataAccess ) {
		this.dataAccess = dataAccess;
	}



	public void buildCustomers() {
		if( dataAccess.getCustomerData().count() <= 0 ) {
			cEric = new Customer( "C86516", "Eric Schulz-Mueller", "eric2346@gmail.com" );
			cAnne = new Customer( "C64327", "Meyer, Anne", "+4917223524" );
			cNadine = new Customer( "C12396", "Nadine Ulla Blumenfeld", "+4915292454" );

			/*
			 * Additional orders from task B1(4). Not part of handout.
			 */
			cTimo = new Customer( "C16865", "Timo Werner", "tw@gmail.com" );
			cSandra = new Customer( "C66865", "Sandra Mueller", "samue62@gmx.de" );

			dataAccess.getCustomerData().saveAll( List.of( cEric, cAnne, cNadine, cTimo, cSandra ) );
		}
	}


	public void buildArticles() {
		if( dataAccess.getArticleData().count() <= 0 ) {
			a1 = new Article( "SKU-458362", "Tasse", 299, 2000 );
			a2 = new Article( "SKU-693856", "Becher", 149, 8400 );
			a3 = new Article( "SKU-518957", "Kanne", 2000, 2400 );
			a4 = new Article( "SKU-638035", "Teller", 649, 7000 );
			a5 = new Article( "SKU-386035", "Kaffeemaschine", 2999, 500 );
			a6 = new Article( "SKU-443803", "Teekocher", 1999, 2000 );

			adjustPrice( a1, priceTable );
			adjustPrice( a2, priceTable );
			adjustPrice( a3, priceTable );
			adjustPrice( a4, priceTable );
			adjustPrice( a5, priceTable );
			adjustPrice( a6, priceTable );

			dataAccess.getArticleData().saveAll( List.of( a1, a2, a3, a4, a5, a6 ) );
		}
	}



	public void buildOrders( Consumer<Order> fillOrder ) {

		// Eric's 1st order
		Order o5234 = new Order( 5234968294L, new Date(), cEric );
		OrderItem oi1 = new OrderItem( a3.getDescription(), a3, 1 );	// 1 Kanne (a3)
		o5234.addItem( oi1 );	// add OrderItem to Order 5234968294L

		// Eric's 2nd order
		Order o8592 = new Order( 8592356245L, new Date(), cEric );
		o8592.addItem(	// add three OrderItems to Order 8592356245L
			new OrderItem( a4.getDescription(), a4, 4 )		// 4 Teller (a4)
		).addItem(
			new OrderItem( a2.getDescription(), a2, 8 )		// 8 Becher (a2)
		).addItem(
			new OrderItem( "passende Tassen", a1, 4 )		// 4 passende Tassen (a1)
		);

		// Anne's order
		Order o3563 = new Order( 3563561357L, new Date(), cAnne );
		o3563.addItem(
			new OrderItem( a3.getDescription() + " aus Porzellan", a3, 1 )
		);

		// Nadine's order
		Order o6135 = new Order( 6135735635L, new Date(), cNadine );
		o6135.addItem(	// 12 Teller
			new OrderItem( a4.getDescription() + " blau/weiss Keramik", a4, 12 )
		);

		Order o9235 = new Order( 9235856245L, new Date(), cTimo );
		o9235.addItem(
			new OrderItem( a5.getDescription(), a5, 1 )		// 1 Kaffeemaschine (a5)
		).addItem(
			new OrderItem( a1.getDescription(), a1, 6 )		// 4 Tassen (a1)
		);
		Order o7335 = new Order( 7335856245L, new Date(), cSandra );
		o7335.addItem(
			new OrderItem( a6.getDescription(), a6, 1 )		// 1 Teekocher (a6)
		).addItem(
			new OrderItem( a2.getDescription(), a2, 4 )		// 4 Becher (a2)
		).addItem(
			new OrderItem( a4.getDescription(), a4, 4 )		// 4 Teller (a4)
		);

		// Need to track orders
		fillOrder.accept( o5234 );
		fillOrder.accept( o8592 );
		fillOrder.accept( o3563 );
		fillOrder.accept( o6135 );
		fillOrder.accept( o9235 );
		fillOrder.accept( o7335 );

		// build Nadine's 2nd order from existing data
		Order o7356 = new Order( 7356613535L, new Date(), cNadine );
		//Article teller = data.getArticle( 3 );

		dataAccess.getArticleData().findById( "SKU-638035" /*SKU for Teller*/ ).ifPresent(teller -> {
			o7356.addItem(	// 6 Teller
				new OrderItem( teller.getDescription() + " blau/weiss Keramik", teller, 6 )
			);
			fillOrder.accept( o7356 );
		});

		dataAccess.getOrderData().saveAll( List.of( o5234,  o8592,  o3563,  o6135, o9235, o7335, o7356 ) );
	}

	private Optional<MockDataBuilderImpl> mockDataBuilder;

	@Override
	public Optional<MockDataBuilderImpl> buildMockData() {
			if( !mockDataBuilder.isPresent() ) {
				mockDataBuilder = Optional.of( new MockDataBuilderImpl((DataAccess) this) );
			}
			return mockDataBuilder;

	}




	/*
	 * Private methods.
	 */
	private void adjustPrice( Article article, Object[][] priceTable ) {
		String id = article.getId();
		for( Object[] line : priceTable ) {
			String sku = (String)line[ 0 ];
			if( sku.equals( id ) ) {
				long price = (int)line[ 1 ];
				article.setUnitPrice( price );
			}
		}
	}

}
